#include <DHT22.h>
#include <SD.h>

#define PIN_DHT_E 4    // pin number for external DHT22 sensor
#define PIN_DHT_I 7    // pin number for internal DHT22 sensor
#define PIN_FET   2      // pin number for MOSFET 
#define interval  60000   // how many milliseconds to wait between iterations
#define x_start   587    // minimum battery voltage to start fan
#define x_stop    570    // maximum battery voltage to stop fan
#define x_light   400    // minimum brightness during daylight

int v_bat = 0;
int v_sol = 0;
int i_lux = 0;
int i_tmp = 0;
int i_hum = 0;
int i_err = 0;
int i_fan = 0;
int e_lux = 0;
int e_ir = 0;
int e_chk = 0;
int e_tmp = 0;
int e_hum = 0;


// Setup a DHT22 instance
DHT22 myDHT22_I(PIN_DHT_I);
DHT22 myDHT22_E(PIN_DHT_E);


void setup() {
 pinMode(PIN_FET, OUTPUT);
 delay(20);
 digitalWrite(PIN_FET, HIGH);
 delay(1000);
 digitalWrite(PIN_FET, LOW);
 Serial.begin(9600);
 delay(1000); 
 
   Serial.print("Initializing SD card...");
  // make sure that the default chip select pin is set to
  // output, even if you don't use it:
  pinMode(10, OUTPUT);

  
  // see if the card is present and can be initialized:
  if (!SD.begin(10)) {
    Serial.println("Card failed, or not present");
    // don't do anything more:
    return;
  }
  Serial.println("card initialized.");
 CaCuTitles();
}

void loop () {
 
 e_lux = analogRead(A0);
 Serial.println(e_lux);
 delay(20);
 
 e_chk = analogRead(A1);
 Serial.println(e_chk);
 delay(20);
  
 e_ir = analogRead(A2);
 Serial.println(e_ir);
 delay(20); 
  
 i_lux = analogRead(A5);
 Serial.println(i_lux);
 delay(20);
 
 v_sol = analogRead(A6);
 Serial.println(v_sol);
 delay(20);
 
 v_bat = analogRead(A7);
 Serial.println(v_bat); 
 Serial.println();
 
 CaCuTestDHTInt();
 CaCuTestDHTExt(); 
 
 CaCuCard();
 
 // check if the fan should be turned on
 
 if ((v_bat > x_start) && (i_lux > x_light)) {
   digitalWrite(PIN_FET, HIGH);
   i_fan = 1; 
 }
 
 // check if the fan should be turned off
 
 if ((v_bat < x_stop) || (i_lux < x_light)) {
   digitalWrite(PIN_FET, LOW);
   i_fan = 0;
 }
 
 
 delay(interval);
}

void CaCuTestDHTInt() {    
      // DHT code begins      
      DHT22_ERROR_t errorCode; 
      errorCode = myDHT22_I.readData();
      switch(errorCode)
      {
        case DHT_ERROR_NONE:
          i_tmp = (myDHT22_I.getTemperatureC());
          i_hum = (myDHT22_I.getHumidity());
          break;
        case DHT_ERROR_CHECKSUM:
          Serial.print("check sum error,?,");
          break;
        case DHT_BUS_HUNG:
          Serial.print("BUS Hung,?,");
          break;
        case DHT_ERROR_NOT_PRESENT:
          Serial.print("Not Present,?,");
          break;
        case DHT_ERROR_ACK_TOO_LONG:
          Serial.print("ACK time out,?,");
          break;
        case DHT_ERROR_SYNC_TIMEOUT:
          Serial.print("Sync Timeout,?,");
          break;
        case DHT_ERROR_DATA_TIMEOUT:
          Serial.print("Data Timeout,?,");
          break;
        case DHT_ERROR_TOOQUICK:
          Serial.print("Polled to quick,?,");
          break;
      }
}

void CaCuTestDHTExt() {    
      // DHT code begins      
      DHT22_ERROR_t errorCode; 
      errorCode = myDHT22_E.readData();
      switch(errorCode)
      {
        case DHT_ERROR_NONE:
          e_tmp = (myDHT22_E.getTemperatureC());
          e_hum = (myDHT22_E.getHumidity());
          break;
        case DHT_ERROR_CHECKSUM:
          Serial.print("check sum error,?,");
          break;
        case DHT_BUS_HUNG:
          Serial.print("BUS Hung,?,");
          break;
        case DHT_ERROR_NOT_PRESENT:
          Serial.print("Not Present,?,");
          break;
        case DHT_ERROR_ACK_TOO_LONG:
          Serial.print("ACK time out,?,");
          break;
        case DHT_ERROR_SYNC_TIMEOUT:
          Serial.print("Sync Timeout,?,");
          break;
        case DHT_ERROR_DATA_TIMEOUT:
          Serial.print("Data Timeout,?,");
          break;
        case DHT_ERROR_TOOQUICK:
          Serial.print("Polled to quick,?,");
          break;
      }
}


void CaCuTitles(){
  // open the file. note that only one file can be open at a time,
  // so you have to close this one before opening another.
  File dataFile = SD.open("shed.csv", FILE_WRITE);

  // if the file is available, write to it:
  if (dataFile) {
    dataFile.print("e_lux");
    dataFile.print(",");
    dataFile.print("e_chk");
    dataFile.print(",");
    dataFile.print("e_ir");
    dataFile.print(",");
    dataFile.print("e_tmp");
    dataFile.print(",");
    dataFile.print("e_hum");
    dataFile.print(",");
    dataFile.print("v_bat");
    dataFile.print(",");
    dataFile.print("v_sol");
    dataFile.print(",");
    dataFile.print("i_lux");
    dataFile.print(",");
    dataFile.print("i_tmp");
    dataFile.print(",");
    dataFile.print("i_hum");
    dataFile.print(",");
    dataFile.print("i_err");
    dataFile.print(",");
    dataFile.print("i_fan");
    dataFile.print(",");
    dataFile.println();
    dataFile.close();
  }  
  // if the file isn't open, pop up an error:
  else {
    Serial.println("error opening datalog.txt");
  }   
}

void CaCuCard(){
  // open the file. note that only one file can be open at a time,
  // so you have to close this one before opening another.
  File dataFile = SD.open("shed.csv", FILE_WRITE);

  // if the file is available, write to it:
  if (dataFile) {
    dataFile.print(e_lux);
    dataFile.print(",");
    dataFile.print(e_chk);
    dataFile.print(",");
    dataFile.print(e_ir);
    dataFile.print(",");
    dataFile.print(e_tmp);
    dataFile.print(",");
    dataFile.print(e_hum);
    dataFile.print(",");
    dataFile.print(v_bat);
    dataFile.print(",");
    dataFile.print(v_sol);
    dataFile.print(",");
    dataFile.print(i_lux);
    dataFile.print(",");
    dataFile.print(i_tmp);
    dataFile.print(",");
    dataFile.print(i_hum);
    dataFile.print(",");
    dataFile.print(i_err);
    dataFile.print(",");
    dataFile.print(i_fan);
    dataFile.print(",");
    dataFile.println();
    dataFile.close();
  }  
  // if the file isn't open, pop up an error:
  else {
    Serial.println("error opening datalog.txt");
  }   
}
